use crate::spotify::ContentKind;
use crate::youtube::Video;
use crate::{spotify, youtube, TrackInfo};

#[derive(Debug, Clone)]
pub(crate) struct TrackItem {
    spotify_track: Option<TrackInfo>,
    youtube_track: Option<Vec<Video>>,
}

// The enum holds all the currently supported type of Id which the engine can search for
#[derive(Debug, Clone)]
pub(crate) enum ServiceIdKind {
    Spotify(String),
    Youtube(String),
    Automatic(String),
}

// This struct will allow us in the future to search, cache and store data and metadata regarding
// tracks, albums and playlists
#[derive(Debug, Clone)]
pub(crate) struct MusicEngine {
    spotify: spotify::Client,
    youtube: youtube::Client,
}

impl MusicEngine {
    pub(crate) async fn new() -> Self {
        MusicEngine {
            spotify: spotify::Client::new().await,
            youtube: youtube::Client::new().await,
        }
    }

    pub(crate) fn new_with_dependencies(
        spotify_client: spotify::Client,
        youtube_client: youtube::Client,
    ) -> Self {
        MusicEngine {
            spotify: spotify_client,
            youtube: youtube_client,
        }
    }

    pub(crate) async fn search_song_by_name(&self, name: &str) {
        todo!("In the future it would be possible to search for all metadata on a record from this call")
    }

    pub(crate) async fn search_song_by_id(&self, id: &str) -> Option<TrackItem> {
        let entry_kind = spotify::get_entry_kind(id);
        let track_info = match entry_kind {
            Some(entry) => match entry {
                ContentKind::Track(id) => self.spotify.get_track(id.as_str()).await,
                _ => None,
            },
            None => None,
        };

        if track_info.is_some() {
            let ti = track_info.unwrap();
            let youtube_search = match self
                .youtube
                .search_video(
                    format!(
                        "{} {}",
                        ti.artists
                            .get(0)
                            .map(|artist| format!("{} -", artist))
                            .unwrap_or("".to_string()),
                        ti.name
                    )
                    .as_str(),
                    None,
                )
                .await
            {
                Err(_) => None,
                Ok(search) => Some(search),
            };

            return Some(TrackItem {
                spotify_track: Some(ti),
                youtube_track: youtube_search.map(|search| search.items),
            });
        }
        return None;
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn should_search_track_by_spotify_id() {
        todo!("Implement me!")
    }
}
